/*
 * Copyright (c) 2019 QNX Software Systems. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _J7_CPSW_H_
#define _J7_CPSW_H_

#include <io-pkt/iopkt_driver.h>
#include <stdio.h>
#include <errno.h>
#include <atomic.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/siginfo.h>
#include <sys/syspage.h>
#include <sys/neutrino.h>
#include <sys/mbuf.h>
#include <sys/slogcodes.h>
#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <net/if_ether.h>
#include <net/if_media.h>
#include <sys/io-pkt.h>
#include <sys/cache.h>
#include <sys/callout.h>
#include <sys/device.h>
#include <hw/inout.h>
#include <netdrvr/mdi.h>
#include <netdrvr/nicsupport.h>
#include <hw/nicinfo.h>
#include <sys/device.h>
#define _STDDEF_H_INCLUDED
#include <siglock.h>
#include <dev/mii/miivar.h>
#include <sys/trace.h>
#include <device_qnx.h>
#include <net/netbyte.h>

#ifdef  __cplusplus
 extern "C" {
#endif

/******************************************************************************
 *  Defines.
 *****************************************************************************/
#define DEBUG_TRACE                     0x00000100
#define DEBUG_BUFFER                    0x00000200
#define DEBUG_START_PACKET              0x00000080
#define DEBUG_PTP                       0x00000040
#define DEBUG_PTP_APP                   0x00000020
#define DEBUG_MASK                      0x000003FF

#if defined (CPSW2G)
  /* mcu_cpsw_nuss register address definitions */
  #define J721E_CPSW_BASE         0x46000000
  #define J721E_CPSW_SIZE         0x200000
#elif defined (CPSW9G)
  /* main cpsw_nuss register address definitions */
  #define J721E_CPSW_BASE         0x0C000000
  #define J721E_CPSW_SIZE         0x200000
#else
  #error "Unsupported!"
#endif 

#define CPSW_QUIESCE_PULSE      _PULSE_CODE_MINAVAIL
#define CPSW_RX_PULSE          (CPSW_QUIESCE_PULSE + 1)

#define PHY_CALLOUT_DELAY       100


/******************************************************************************
 *  Structs.
 *****************************************************************************/

struct cpsw_dev {
    struct device       sc_dev; /* common device */
    struct ethercom     sc_ec;  /* common ethernet */
    nic_config_t        cfg;    /* nic information */
    nic_stats_t         stats;  /* stats structure */

    struct callout      phy_callout;

    /* whatever else you need follows */
    struct _iopkt_self  *sc_iopkt;
    int                 sc_iid;
    int                 sc_irq;
    int                 sc_intr_cnt;
    int                 sc_intr_spurious;
    struct _iopkt_inter sc_inter;
    void                *sc_sdhook;

    int                 tid;
    int                 chid;
    int                 coid;
    struct sigevent     isr_event;

    int                 mac_to_mac;
    int                 speed;
    int                 ptp_enable;

    int                 dying;
    int                 force_link;
    int                 linkup;
    struct mii_data     bsd_mii;
    uint8_t             ipaddr[4];
};

typedef struct {
    struct _iopkt_self  *iopkt;
    char                *options;
    nic_config_t        cfg;

    uintptr_t           cpsw_base;

    void                *sd_hook;
} attach_args_t;

/******************************************************************************
 *  Functions.
 *****************************************************************************/

/* bsd_media.c */
void bsd_mii_initmedia(struct cpsw_dev *);
void bsd_mii_finimedia(struct cpsw_dev *);
int bsd_mii_mediachange(struct ifnet *ifp);

/* j7_cpsw.c */
int cpsw_entry(void *dll_hdl, struct _iopkt_self *iopkt, char *options);
int cpsw_init(struct ifnet *);
void cpsw_stop(struct ifnet *, int);
void cpsw_start(struct ifnet *);
int cpsw_ioctl(struct ifnet *, unsigned long, caddr_t);
void cpsw_shutdown(void *);
void cpsw_MonitorPhy (void *arg);
int cpsw_process_rx1(void *arg, struct nw_work_thread   *wtp);
struct mbuf * cpsw_process_rx(void *arg, struct mbuf *);
void *cpsw_rx_thread(void *arg);

#ifdef __cplusplus
};
#endif

#endif //_J7_CPSW_H_
